import allegro;

from OpenGL.GL import *;
from OpenGL.GLU import *;

import math;

from GLSLRenderer import GLSLRenderer;



import collada;
import data;


renderers = {};
fonts = {};

def draw_environment(hexagon_grid, current_hexagon):
    glColor(1.0, 0.0, 0.0);
    
    glDisable(GL_LIGHTING);

    for hexagon in hexagon_grid.neighbours(current_hexagon) + [current_hexagon]:
        glBegin(GL_LINE_STRIP);
        for x, y in hexagon.corners():
            
            glVertex(x, y, -40.);
            #print x, y,-20.
        glEnd();
    
    glEnable(GL_LIGHTING);


    for hexagon in hexagon_grid.neighbours(current_hexagon) + [current_hexagon]:
        if hexagon.floor == None:
            floor_render = renderers.get("floor", None);
            if floor_render == None:
                floor_render = GLSLRenderer(collada.Collada(data.filepath("floor.dae"), 
                    ignore=[collada.DaeUnsupportedError, collada.DaeBrokenRefError]));
                renderers["floor"] = floor_render;
        elif hexagon.floor == "semi_walled":
            floor_render = renderers.get("semi_walled", None);
            if floor_render == None:
                floor_render = GLSLRenderer(collada.Collada(data.filepath("semi_walled_floor.dae"), 
                    ignore=[collada.DaeUnsupportedError, collada.DaeBrokenRefError]));
                renderers["semi_walled"] = floor_render;
                
        floor_render.render(hexagon.x, hexagon.y, -25, 0, 0, 0);
        for obj in hexagon.objects:
            if obj.type == "tree":
                draw_tree(obj);
            elif obj.type == "minion":
                draw_minion(obj);
            elif obj.type == "character":
                draw_character(obj);
            elif obj.type == "beast":
                draw_beast(obj);
    
    
    
def draw_character(avatar):
    avatar_renderer = renderers.get(avatar, None);
    if avatar_renderer == None:
        avatar_renderer = GLSLRenderer(collada.Collada(data.filepath("avatar.dae"), 
            ignore=[collada.DaeUnsupportedError, collada.DaeBrokenRefError]));
        renderers[avatar] = avatar_renderer;
        
    avatar_renderer.render(avatar.x, avatar.y, -20, 0, 0, avatar.facing*180.0/math.pi);
    glDisable(GL_LIGHTING);
    
    hitpoint_ratio = avatar.hitpoints/30.;
    
    glColor(0.5-0.5*hitpoint_ratio, 0.5 * hitpoint_ratio, 0);
    glBegin(GL_TRIANGLE_STRIP);
    glVertex(avatar.x -1. + 2. * hitpoint_ratio, avatar.y - 1.5, -15.);
    glVertex(avatar.x - 1., avatar.y - 1.5, -15.);
    glVertex(avatar.x -1. + 2. * hitpoint_ratio, avatar.y - 1.7, -15.);
    glVertex(avatar.x - 1., avatar.y - 1.7, -15.);
    glEnd();
    
    glColor(1, 0, 0);

    glBegin(GL_LINE_STRIP);
    base = avatar.sword.base();
    glVertex(avatar.x + base[0], avatar.y + base[1],  -19.5);
    tip = avatar.sword.tip();
    glVertex(avatar.x+tip[0], avatar.y+tip[1], -19.5);
    glEnd();
    glEnable(GL_LIGHTING);
    

    

def draw_minion(avatar):
    avatar_renderer = renderers.get("minion", None);
    if avatar_renderer == None:
        avatar_renderer = GLSLRenderer(collada.Collada(data.filepath("avatar.dae"), 
            ignore=[collada.DaeUnsupportedError, collada.DaeBrokenRefError]));
        renderers["minion"] = avatar_renderer;
        
    avatar_renderer.render(avatar.x, avatar.y, -20, 0, 0, avatar.facing*180.0/math.pi);
    
    glDisable(GL_LIGHTING);
    glBegin(GL_LINE_STRIP);
    base = avatar.sword.base();
    glVertex(avatar.x + base[0], avatar.y + base[1],  -19.5);
    tip = avatar.sword.tip();
    glVertex(avatar.x+tip[0], avatar.y+tip[1], -19.5);
    glEnd();
    glEnable(GL_LIGHTING);

def draw_beast(avatar):
    avatar_renderer = renderers.get(avatar, None);
    if avatar_renderer == None:
        avatar_renderer = GLSLRenderer(collada.Collada(data.filepath("beast.dae"), 
            ignore=[collada.DaeUnsupportedError, collada.DaeBrokenRefError]));
        renderers[avatar] = avatar_renderer;
    avatar_renderer.render(avatar.x, avatar.y, -15, 0, 0, avatar.facing*180.0/math.pi);

    glDisable(GL_LIGHTING);
    hitpoint_ratio = avatar.hitpoints/200.;
    glColor(0.5-0.5*hitpoint_ratio, 0.5 * hitpoint_ratio, 0);
    glBegin(GL_TRIANGLE_STRIP);
    glVertex(avatar.x -3. + 6. * hitpoint_ratio, avatar.y - 4.3, -15.);
    glVertex(avatar.x - 3., avatar.y - 4.3, -15.);
    glVertex(avatar.x -3. + 6. * hitpoint_ratio, avatar.y - 4.7, -15.);
    glVertex(avatar.x - 3., avatar.y - 4.7, -15.);
    glEnd();
    
    glEnable(GL_LIGHTING);
    
    glColor(1, 0, 0);

        
def draw_tree(tree):
    tree_renderer = renderers.get("tree", None);
    if tree_renderer == None:
        tree_renderer = GLSLRenderer(collada.Collada(data.filepath("tree1.dae"), 
            ignore=[collada.DaeUnsupportedError, collada.DaeBrokenRefError]));
        renderers["tree"] = tree_renderer;
    
    tree_renderer.render(tree.x, tree.y, -20, 0, 0, tree.facing*180.0/math.pi);
    
def draw_line_between_beast_and_player(beast, avatar):
    glDisable(GL_LIGHTING);
    glColor(0., 1., 0.);
    glBegin(GL_LINE_STRIP);
    glVertex(avatar.x, avatar.y,  -19.5);
    tip = avatar.sword.tip();
    glVertex(beast.x, beast.y, -19.5);
    glEnd();
    glEnable(GL_LIGHTING);
    
def draw_right_arrow(avatar):
    glDisable(GL_LIGHTING);
    glColor(0., 1., 0.);
    glBegin(GL_TRIANGLE_STRIP);
    glVertex(avatar.x + 9., avatar.y + 0.5, -15.);
    glVertex(avatar.x + 6., avatar.y + 0.5, -15.);
    glVertex(avatar.x  + 9., avatar.y - 0.5, -15.);
    glVertex(avatar.x + 6., avatar.y - 0.5, -15.);
    glEnd();
    glBegin(GL_TRIANGLE_STRIP);
    glVertex(avatar.x + 10., avatar.y, -15.);
    glVertex(avatar.x + 9., avatar.y + 1., -15.);
    glVertex(avatar.x  + 9., avatar.y - 1., -15.);
    glEnd();
    glEnable(GL_LIGHTING);

def draw_backstory(display, list_of_text):
    # ugh, this stuff is a pain
    # Set projection
    text_font = fonts.get("purisa", None);
    if text_font == None:
        text_font = allegro.al_load_ttf_font(data.filepath("Purisa.ttf"), 40, 12, 0);
        fonts["purisa"] = text_font;
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, allegro.al_get_display_width(display), allegro.al_get_display_height(display), 0, -1, 1);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST) # Enables Depth Testing
    glDisable(GL_CULL_FACE)
    glDisable(GL_MULTISAMPLE);
    
    j_total = 0.;
    for i, text in enumerate(list_of_text):
        for j, line in enumerate(text.split("\n")):
            j_total += 1.;
            allegro.al_draw_text(text_font, allegro.al_map_rgb_f(1.0, 0., 0.), 50., float(i) * 75. + j_total * 50., 0, line);
            

    glEnable(GL_LIGHTING);
    glEnable(GL_DEPTH_TEST) # Enables Depth Testing
    glEnable(GL_CULL_FACE)
    glEnable(GL_MULTISAMPLE);
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(60., float(allegro.al_get_display_width(display)) / float(allegro.al_get_display_height(display)), .1, 1000.)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
