
import math;
import fractions;



class Hexagon:
    def __init__(self, x, y, radius, ring, segment):
        self.x = x;
        self.y = y;
        self.radius = radius;
        self.ring = ring;
        self.segment = segment;
        
    def corners(self):
        points = [];
        for i in range(6):
            angle = (i*60. + 30.)*math.pi/180.0;
            points.append((self.x + self.radius * math.cos(angle), self.y + self.radius*math.sin(angle)));
        return points;
        
    def is_point_inside(self, x, y):
        # circle test
        dist_sq = (x - self.x)**2. + (y - self.y)**2.;
        if dist_sq < self.radius**2.:
            """
            # passed circle test, now for more strict test
            angle = math.atan2(y - self.y, x - self.x) + 2. * math.pi;
            
            # nearest corners
            angle_small = int(angle * 3. / math.pi) * math.pi / 3.;
            angle_large = angle_small + math.pi / 3.;
            
            # find the points that are those corners
            small_point = (self.radius * math.cos(angle_small), self.radius * math.sin(angle_small));
            large_point = (self.radius * math.cos(angle_large), self.radius * math.sin(angle_large));
            
            y1 = small_point[1];
            x1 = small_point[0];
            
            y2 = self.y;
            x2 = self.x;
            
            # line rise over run
            m1 = (large_point[1] - small_point[1])/(large_point[0] - small_point[0]);
            
            m2 = (y - self.y)/(x - self.x);
            
            solution_x = (-y2+y1+m2*x2-m1*x1)/(m1-m2);
            solution_y = (m1*(m2*x2-y2)+m2*y1-m1*m2*x1)/(m1-m2);
            """
            solution_x, solution_y = self.get_closest_point(x, y);
            soln_dist_sq = (solution_x - self.x)**2. + (solution_y - self.y)**2.;
            
            if dist_sq < soln_dist_sq:
                return True;
            
        return False;
        
    def get_closest_point(self, x, y):
        angle = math.atan2(y - self.y, x - self.x) + 2. * math.pi;
        
        # nearest corners
        angle_small = float(int((angle + math.pi/6.) * 3. / math.pi)) * math.pi / 3. - math.pi/6.;
        angle_large = angle_small + math.pi / 3.;
        
        # find the points that are those corners
        small_point = (self.radius * math.cos(angle_small) + self.x, self.radius * math.sin(angle_small) + self.y);
        large_point = (self.radius * math.cos(angle_large) + self.x, self.radius * math.sin(angle_large) + self.y);
        
        y1 = small_point[1];
        x1 = small_point[0];
        
        #print "small_point", small_point;
        #print "large_point", large_point;
        
        y2 = self.y;
        x2 = self.x;
        
        if large_point[0] - small_point[0] == 0.:
            return (large_point[0], y);
        if large_point[1] - small_point[1] == 0.:
            return (x, large_point[1]);
        else:
            # line rise over run
            m1 = (large_point[1] - small_point[1])/(large_point[0] - small_point[0]);
            
            m2 = (y - self.y)/(x - self.x);
            
            solution_x = -(-y2+y1+m2*x2-m1*x1)/(m1-m2);
            solution_y = -(m1*(m2*x2-y2)+m2*y1-m1*m2*x1)/(m1-m2);
        
        return (solution_x, solution_y);
        
        
        
class HexagonGrid:
    def __init__(self, radius):
        self.radius = radius;
        self.radius_multiplier = math.sqrt(3) * self.radius;
        self.hexagons = {};
        self.hexagons_by_position = {};
        self.hexagon_neighbours = {};
        
    def lookup_hexagon(self, x, y):
        x_str = "%.2f" % x;
        y_str = "%.2f" % y;
        
        if x_str == "-0.00":
            x_str = "0.00";
        
        if y_str == "-0.00":
            y_str = "0.00";
        
        key = x_str+":"+y_str;
        ret = self.hexagons_by_position.get(key, None);
        return ret;
        
    def mag_angle(self, ring, segment):
        if ring == 0:
            return (0, 0);
        elif ring < 2 or segment % ring == 0:
            return (self.radius_multiplier * ring, float(segment) * 2. * math.pi / (float(ring)*6.));
        else:
            # these hexagons lie between
            smaller_side = float(int(float(segment) / ring) * ring);
            bigger_side = smaller_side + ring;
            
            mag_smaller_side, angle_smaller_side = self.mag_angle(ring, smaller_side);
            mag_bigger_side, angle_bigger_side = self.mag_angle(ring, bigger_side);
                        
            center_smaller_side = (mag_smaller_side * math.cos(angle_smaller_side), mag_smaller_side * math.sin(angle_smaller_side));           
            center_bigger_side = (mag_bigger_side * math.cos(angle_bigger_side), mag_bigger_side * math.sin(angle_bigger_side));
            
            ratio = (float(segment) - smaller_side)/float(bigger_side - smaller_side);
            
            new_point = (center_smaller_side[0] + (center_bigger_side[0] - center_smaller_side[0]) * ratio, 
                center_smaller_side[1] + (center_bigger_side[1] - center_smaller_side[1]) * ratio);
            
            return (math.sqrt(new_point[0]*new_point[0] + new_point[1]*new_point[1]), math.atan2(new_point[1], new_point[0]));
            
       
    def segment(self, mag, angle):
        return angle * self.ring(mag, angle) / (2. * math.pi);
            
    def get(self, ring, segment, create = True):
        if segment > ring * 6:
            return None;
        key = str(ring)+":"+str(segment);
        h = self.hexagons.get(key, None);
        if h == None and create:
            mag, angle = self.mag_angle(ring, segment);
            h = Hexagon(mag * math.cos(angle), mag * math.sin(angle), self.radius, ring, segment);
            self.hexagons[key] = h;
            x_str = "%.2f" % h.x;
            y_str = "%.2f" % h.y;
            if x_str == "-0.00":
                x_str = "0.00";
            if y_str == "-0.00":
                y_str = "0.00";
            pos_key = x_str+":"+y_str;
            self.hexagons_by_position[pos_key] = h;
        return h;
        
    def neighbours(self, h):
        # generate and cache the neighbours
        key = str(h.ring)+":"+str(h.segment);
        neighbours_list = self.hexagon_neighbours.get(key,[]);
        if len(neighbours_list) == 0:
            self.hexagon_neighbours[key] = neighbours_list;
        elif len(neighbours_list) == 6:
            return neighbours_list;
        
        # always have 6 neighbours
        for i in range(6):
            n_x = h.x + self.radius_multiplier*math.cos(float(i)*60.*math.pi/180.);
            n_y = h.y + self.radius_multiplier*math.sin(float(i)*60.*math.pi/180.);
            
    
            
            n = self.lookup_hexagon(n_x, n_y);
            
        
            if n != None and n not in neighbours_list:
                neighbours_list.append(n);
        return neighbours_list;
                

if __name__ == "__main__":
    print "main";
    from pylab import *;
    hg = HexagonGrid(10.);

    # center
    hg.get(0, 0);

    # ring 1
    for i in range(6):
        hg.get(1, i);

    # ring 2
    for i in range(12):
        hg.get(2, i);
    
    #hg.get(2, 0);
    #hg.get(2, 2);
    #hg.get(2, 4);
    #hg.get(2, 6);
    #hg.get(2, 8);
    #hg.get(2, 10);
    #hg.get(2, 12);
    
    #hg.get(3, 0);
    #hg.get(3, 3);
    
    # ring 3
    for i in range(18):
        hg.get(3, i);

    # ring 4
    for i in range(6*4):
        hg.get(4, i);

    colors = iter(['r', 'g', 'b', 'c', 'k', 'y', 'm']*10);

    for k, h in hg.hexagons.items():
        color = colors.next();
        corners_iter = iter(h.corners());
        prev_x, prev_y = corners_iter.next();
        for corner_x, corner_y in corners_iter:
            plot([prev_x, corner_x], [prev_y, corner_y], color+'-');
            prev_x, prev_y = corner_x, corner_y;

        for neighbour in hg.neighbours(h):
            plot([h.x, neighbour.x], [h.y, neighbour.y], 'k-');
            
    show();
