
import allegro;
import main;
import sys;
import data;

from OpenGL.GL import *;
from OpenGL.GLU import *;

class Menu:
    def __init__(self, config):
        
        self.main_menu = [];
        self.options_menu = [];
        self.graphics_menu = [];
        self.sound_menu = [];
        self.controls_menu = [];
        self.resolutions_menu = [];
        self.full_screen_menu = [];
        self.restart_required = [("RESTART REQUIRED FOR", 
            self.switch_to(self.graphics_menu)),
            ("CHANGES TO TAKE EFFECT", self.switch_to(self.graphics_menu))];
        
        self.main_menu.extend(
        [
            ("Play", self.hide),
            ("Options", self.switch_to(self.options_menu)),
            ("Exit", self.exit)
        ]);
        
        self.options_menu.extend(
        [
            ("Graphics", self.switch_to(self.graphics_menu)),
            ("Sound", self.switch_to(self.sound_menu)),
            ("Controls", self.switch_to_controls_menu),
            ("Back", self.switch_to(self.main_menu))
        ]);
        
        self.graphics_menu.extend(
        [
           ("Resolution", self.switch_to(self.resolutions_menu)),
           ("Fullscreen", self.switch_to(self.full_screen_menu)),
           ("Back", self.switch_to(self.options_menu))
        ]);
        
        self.sound_menu.extend(
        [
            ("Back", self.switch_to(self.options_menu))
        ]);
        
        self.controls_menu.extend(
        [
           ("Back", self.switch_to(self.options_menu))
        ]);

        self.current_menu = None;
        self.selected = 0;
        
        self.visible = False;
        
        self.title = allegro.al_load_bitmap(data.filepath("title.png"));
        self.tint = 1;
        self.fade_out = False;
        
        self.menu_font = allegro.al_load_ttf_font(data.filepath("Purisa.ttf"), 40, 12, 0);
        
        num_display_modes = allegro.al_get_num_display_modes();
        
        for i in range(num_display_modes):
            dm = allegro.ALLEGRO_DISPLAY_MODE();
            allegro.al_get_display_mode(i, dm);
            x = (int(dm.width), int(dm.height), int(dm.format), int(dm.refresh_rate));
            self.resolutions_menu.append(("%ix%ix%i@%iHz" % (dm.width, dm.height, dm.format, dm.refresh_rate), lambda x=x: self.switch_resolution(*x)));
        self.resolutions_menu.append(("Back", self.switch_to(self.graphics_menu)));
        
        self.full_screen_menu.extend(
        [
            ("true", lambda x="true": self.set_fullscreen(x)),
            ("false", lambda x="false": self.set_fullscreen(x)),
            ("Back", self.switch_to(self.graphics_menu))
        ]);
        
        self.config = config;
        
        self.standard_key_map =\
        {
            allegro.ALLEGRO_KEY_LSHIFT: "lshift",
            allegro.ALLEGRO_KEY_RSHIFT: "rshift",
            allegro.ALLEGRO_KEY_LCTRL: "lctrl",
            allegro.ALLEGRO_KEY_RCTRL: "rctrl",
            allegro.ALLEGRO_KEY_SPACE: "space",
            allegro.ALLEGRO_KEY_CAPSLOCK: "caps",
            allegro.ALLEGRO_KEY_TAB: "tab",
            allegro.ALLEGRO_KEY_ENTER: "enter",
            allegro.ALLEGRO_KEY_ALT: "alt",
            allegro.ALLEGRO_KEY_COMMAND: "windows/command",
            allegro.ALLEGRO_KEY_UP: "up",
            allegro.ALLEGRO_KEY_DOWN: "down",
            allegro.ALLEGRO_KEY_LEFT: "left",
            allegro.ALLEGRO_KEY_RIGHT: "right"
        };

        self.extra_event_handler = None;
        
        self.restart = False;
    
    def show(self, display, queue):
        print "menu show";
        self.display = display;
        self.queue = queue;
        # ugh, this stuff is a pain
        # Set projection
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glOrtho(0, allegro.al_get_display_width(display), allegro.al_get_display_height(display), 0, -1, 1);

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        
        glDisable(GL_LIGHTING);
        glDisable(GL_DEPTH_TEST) # Enables Depth Testing
        glDisable(GL_CULL_FACE)
        glDisable(GL_MULTISAMPLE);
        
        self.visible = True;
        while self.visible:
            event = allegro.ALLEGRO_EVENT()

            if allegro.al_is_event_queue_empty(queue):
                allegro.al_clear_to_color(allegro.al_map_rgb_f(0.2, 0.2, 0.2));
                
                if self.current_menu == None:
                    # display the title
                    allegro.al_draw_tinted_scaled_bitmap(
                        self.title, 
                        allegro.al_map_rgb_f(self.tint, self.tint, self.tint),
                        0., 0., 1024., 1024.,
                        0., 0.,
                        float(allegro.al_get_display_width(display)),
                        float(allegro.al_get_display_width(display))/1024. * float(allegro.al_get_display_height(display)),
                         0, 0, 0);
                
                else:
                    # draw the menu
                    for i, (item, ignore) in enumerate(self.current_menu):
                        x = int(float(i) / 10);
                        y = i % 10;
                        if i == self.selected:
                            allegro.al_draw_text(self.menu_font, allegro.al_map_rgb_f(1.0, 0., 0.), x*500 + 100, (y+1)*50, 0, item);
                        else:
                            allegro.al_draw_text(self.menu_font, allegro.al_map_rgb_f(0.7, 0., 0.), x*500 + 100, (y+1)*50, 0, item);
                    
                allegro.al_flip_display();
                
            allegro.al_wait_for_event(queue, allegro.byref(event));

            if event.type == allegro.ALLEGRO_EVENT_DISPLAY_CLOSE:
                sys.exit(0);                
               
            if self.current_menu != None:
                if event.type == allegro.ALLEGRO_EVENT_KEY_UP:
                    if event.keyboard.keycode == allegro.ALLEGRO_KEY_UP:
                        self.selected -= 1;
                        if self.selected < 0:
                            self.selected = len(self.current_menu)-1;
                    elif event.keyboard.keycode == allegro.ALLEGRO_KEY_DOWN:
                        self.selected += 1;
                        if self.selected >= len(self.current_menu):
                            self.selected = 0;
                            
                    elif event.keyboard.keycode == allegro.ALLEGRO_KEY_ENTER:
                        self.current_menu[self.selected][1]();
                        
                if self.extra_event_handler != None:
                    self.extra_event_handler(event);
            else:
                # we are on the title screen
                if event.type == allegro.ALLEGRO_EVENT_TIMER:
                    if self.fade_out:
                        self.fade_out_count -= 1;
                        
                        # with fade out set to 180
                        
                        self.tint = float(self.fade_out_count)/180.;
                        if self.fade_out_count == 0:
                            self.fade_out == False;
                            self.current_menu = self.main_menu;
                
                elif event.keyboard.keycode == allegro.ALLEGRO_KEY_ENTER:
					
                    # 60fps maximum, so about 3 secs
                    if not self.fade_out:
						self.fade_out_count = 180;
						
						# start fading out
						self.fade_out = True;
                    

        print "leave show";
                    
    def hide(self):
        self.visible = False;
        if len(self.main_menu) < 4:
            self.main_menu.pop(0);
            self.main_menu.insert(0, ("Resume", self.hide));
            self.main_menu.insert(1, ("Restart", self.set_restart));
        glEnable(GL_LIGHTING);
        glEnable(GL_DEPTH_TEST) # Enables Depth Testing
        glEnable(GL_CULL_FACE)
        glEnable(GL_MULTISAMPLE);
        
        glViewport(0, 0, allegro.al_get_display_width(self.display), 
            allegro.al_get_display_height(self.display))
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(60., float(allegro.al_get_display_width(self.display)) / float(allegro.al_get_display_height(self.display)), .1, 1000.)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        allegro.al_clear_to_color(allegro.al_map_rgb_f(0.1, 0.1, 0.1));
        print "menu hide";
        
    def switch_to(self, menu):
        def switch_delayed():
            self.current_menu = menu;
            self.selected = 0;
        return switch_delayed;
    
    def keycode_to_name(self, keycode):
        if keycode >= allegro.ALLEGRO_KEY_A and keycode <= allegro.ALLEGRO_KEY_Z:
            
            return chr(keycode - allegro.ALLEGRO_KEY_A + ord('a'));
        elif keycode >= allegro.ALLEGRO_KEY_0 and keycode <= allegro.ALLEGRO_KEY_9:
            return chr(keycode - allegro.ALLEGRO_KEY_0 + ord('0'));
        else:
            # not a number or a character, just test for some standard keys
            print str(keycode);
            return self.standard_key_map.get(keycode, str(keycode));
    
    def generate_control_item(self, name):
        return (name + " ... " + self.keycode_to_name(int(allegro.al_get_config_value(self.config, "keys", name))), self.set_key(name));
    
    def set_key(self, name):
        print name;
        def extra_event_handler(event, name=name):
            if event.type == allegro.ALLEGRO_EVENT_KEY_DOWN:
                allegro.al_set_config_value(self.config, "keys", name, str(int(event.keyboard.keycode)));
                allegro.al_save_config_file(data.filepath("config.txt"), self.config);
                self.extra_event_handler = None;
                self.switch_to_controls_menu();
                
        def switch():
            self.extra_event_handler = extra_event_handler;
            self.switch_to([("type key for " + name, lambda: None)])();
        return switch;
    
    def switch_to_controls_menu(self):
            while len(self.controls_menu) > 0:
                self.controls_menu.pop();
            self.controls_menu.append(self.generate_control_item("moving_forward"));
            self.controls_menu.append(self.generate_control_item("moving_backward"));
            self.controls_menu.append(self.generate_control_item("turning_left"));
            self.controls_menu.append(self.generate_control_item("turning_right"));
            
            self.controls_menu.append(self.generate_control_item("attacking"));
            self.controls_menu.append(self.generate_control_item("defending"));
            self.controls_menu.append(self.generate_control_item("jumping"));
            self.controls_menu.append(self.generate_control_item("pausing"));
            self.controls_menu.append(("Back", self.switch_to(self.main_menu)));
            self.switch_to(self.controls_menu)();
        
    def switch_resolution(self, width, height, format, refresh_rate):
        print "switch_resolution", width, height, format, refresh_rate;
        allegro.al_set_config_value(self.config, "graphics", "width", str(width));
        allegro.al_set_config_value(self.config, "graphics", "height", str(height));
        allegro.al_set_config_value(self.config, "graphics", "refresh_rate", str(refresh_rate));
        allegro.al_save_config_file(data.filepath("config.txt"), self.config);
        self.switch_to(self.restart_required)();
        
    def set_fullscreen(self, fullscreen):
        print "set_fullscreen", fullscreen;
        allegro.al_set_config_value(self.config, "graphics", "fullscreen", fullscreen);
        allegro.al_save_config_file(data.filepath("config.txt"), self.config);
        self.switch_to(self.restart_required)();

    def exit(self):
        sys.exit(0);
        
    def set_restart(self):
        self.restart = True;
        self.hide();
