
import allegro;
import utils;
import math;
import random;

class Player:
    def __init__(self, avatar):
        self.avatar = avatar;
        
        self.moving_forward = False;
        self.moving_backward = False;
        self.turning_left = False;
        self.turning_right = False;
            
        self.attacking = False;
        self.defending = False;
        self.jumping = False;
        
        self.speed_forward = 0.2;
        self.speed_backward = -0.2;
        self.omega_left = 0.05;
        self.omega_right = -0.05;
        
        
    def set_moving_forward(self):
        self.moving_forward = True;
    
    def unset_moving_forward(self):
        self.moving_forward = False;
        
    def set_moving_backward(self):
        self.moving_backward = True;
        
    def unset_moving_backward(self):
        self.moving_backward = False;
        
    def set_turning_left(self):
        self.turning_left = True;
        
    def unset_turning_left(self):
        self.turning_left = False;
        
    def set_turning_right(self):
        self.turning_right = True;
        
    def unset_turning_right(self):
        self.turning_right = False;
        
    def set_attacking(self):
        self.attacking = True;
        
    def unset_attacking(self):
        self.attacking = False;
        
    def set_defending(self):
        self.defending = True;
        
    def unset_defending(self):
        self.defending = False;
        
    def set_jumping(self):
        self.jumping = True;
        
    def unset_jumping(self):
        self.jumping = False;
        
    def update(self):
        if self.avatar.hitpoints < 0:
            self.unset_moving_forward();
            self.unset_moving_backward();
            self.unset_turning_left();
            self.unset_turning_right();
            self.unset_attacking();
            self.unset_defending();
            self.unset_jumping();
        
        if self.moving_forward:
            self.avatar.set_speed(self.speed_forward);
        elif self.moving_backward:
            self.avatar.set_speed(self.speed_backward);
        else:
            self.avatar.set_speed(0.);
            
        if self.turning_left:
            self.avatar.set_omega(self.omega_left);
        elif self.turning_right:
            self.avatar.set_omega(self.omega_right);
        else:
            self.avatar.set_omega(0.);
            
        self.avatar.set_attacking(self.attacking);
        
        self.avatar.set_defending(self.defending);
            
        self.avatar.set_jumping(self.jumping);
        

class HumanPlayer(Player):
    def __init__(self, avatar):
        Player.__init__(self, avatar);
        self.key_down_map =\
        {
            allegro.ALLEGRO_KEY_W: self.set_moving_forward,
            allegro.ALLEGRO_KEY_S: self.set_moving_backward,
            allegro.ALLEGRO_KEY_A: self.set_turning_left,
            allegro.ALLEGRO_KEY_D: self.set_turning_right,
            
            allegro.ALLEGRO_KEY_LCTRL: self.set_attacking,
            allegro.ALLEGRO_KEY_LSHIFT: self.set_defending,
            allegro.ALLEGRO_KEY_SPACE: self.set_jumping
        };
        self.key_up_map =\
        {
            allegro.ALLEGRO_KEY_W: self.unset_moving_forward,
            allegro.ALLEGRO_KEY_S: self.unset_moving_backward,
            allegro.ALLEGRO_KEY_A: self.unset_turning_left,
            allegro.ALLEGRO_KEY_D: self.unset_turning_right,
            
            allegro.ALLEGRO_KEY_LCTRL: self.unset_attacking,
            allegro.ALLEGRO_KEY_LSHIFT: self.unset_defending,
            allegro.ALLEGRO_KEY_SPACE: self.unset_jumping
        };
        
    def on_event(self, event):
        if event.type == allegro.ALLEGRO_EVENT_KEY_UP:
            self.key_up_map.get(event.keyboard.keycode, lambda: None)();
        elif event.type == allegro.ALLEGRO_EVENT_KEY_DOWN:
            self.key_down_map.get(event.keyboard.keycode, lambda: None)();
    
    def set_action_key(self, action_down, action_up, key):
        self.key_down_map[key] = action_down;
        self.key_up_map[key] = action_up;
        
    def keys_from(self, config):
        self.key_down_map = {};
        self.key_up_map = {};
        self.set_action_key(self.set_moving_forward, self.unset_moving_forward, int(allegro.al_get_config_value(config, "keys", "moving_forward")));
        self.set_action_key(self.set_moving_backward, self.unset_moving_backward, int(allegro.al_get_config_value(config, "keys", "moving_backward")));
        self.set_action_key(self.set_turning_left, self.unset_turning_left, int(allegro.al_get_config_value(config, "keys", "turning_left")));
        self.set_action_key(self.set_turning_right, self.unset_turning_right, int(allegro.al_get_config_value(config, "keys", "turning_right")));
        
        self.set_action_key(self.set_attacking, self.unset_attacking, int(allegro.al_get_config_value(config, "keys", "attacking")));
        self.set_action_key(self.set_defending, self.unset_defending, int(allegro.al_get_config_value(config, "keys", "defending")));
        self.set_action_key(self.set_jumping, self.unset_jumping, int(allegro.al_get_config_value(config, "keys", "jumping")));
        
                    
        
class BeastAIPlayer(Player):
    def __init__(self, avatar, player):
        Player.__init__(self, avatar);
        self.set_new_target();
        self.state = "goto-target";

    
    def set_new_target(self):
        self.target = (random.random()*600.0 - 300.0 , random.random()*600.0 - 300.0);
                
    def update(self):
        # do something sensible
        if self.state == "goto-target":
            self.unset_attacking();
            angle_betw = math.atan2(self.target[1] - self.avatar.y,
                self.target[0] - self.avatar.x);
            dist_betw = math.sqrt((self.target[0] - self.avatar.x)**2. + (self.target[1] - self.avatar.y)**2.);
            
            if dist_betw < 1:
                # acquire new target!
                self.set_new_target();
            else:
                if utils.shortest_difference_between_angles(angle_betw, self.avatar.facing) > 10. * math.pi / 180.:
                    # turn towards
                    self.unset_moving_forward();
                    if utils.compare_angles(angle_betw, self.avatar.facing) > 0:
                        self.unset_turning_right();
                        self.set_turning_left();
                    else:
                        self.unset_turning_left();
                        self.set_turning_right();
                else:
                    self.unset_turning_left();
                    self.unset_turning_right();
                    self.set_moving_forward();
        elif self.state == "attack":
            angle_betw = math.atan2(self.player1.avatar.y - self.avatar.y,
                self.player1.avatar.x - self.avatar.x);
            dist_betw = math.sqrt((self.player1.avatar.x - self.avatar.x)**2. + (self.player1.avatar.y - self.avatar.y)**2.);
            
            if utils.shortest_difference_between_angles(angle_betw, self.avatar.facing) > 10. * math.pi / 180.:
                # turn towards
                self.unset_moving_forward();
                self.unset_attacking();
                if utils.compare_angles(angle_betw, self.avatar.facing) > 0:
                    self.unset_turning_right();
                    self.set_turning_left();
                else:
                    self.unset_turning_left();
                    self.set_turning_right();
            elif dist_betw > self.avatar.sword.length:
                #charge!
                self.unset_turning_left();
                self.unset_turning_right();
                self.set_attacking();
                self.set_moving_forward();
            else:
                self.unset_turning_left();
                self.unset_turning_right();
                self.unset_moving_forward();
                self.set_attacking();
        Player.update(self);
        
    

class MinionAIPlayer(Player):
    def __init__(self, avatar, player1):
        Player.__init__(self, avatar);
        self.player1 = player1;
        self.speed_forward = 0.1;
        self.speed_backward = -0.1;
        self.omega_left = 0.025;
        self.omega_right = -0.025;
        
    def update(self):
        # do something sensible
        # attack!
        angle_betw = math.atan2(self.player1.avatar.y - self.avatar.y,
            self.player1.avatar.x - self.avatar.x);
        dist_betw = math.sqrt((self.player1.avatar.x - self.avatar.x)**2. + (self.player1.avatar.y - self.avatar.y)**2.);
        
        if utils.shortest_difference_between_angles(angle_betw, self.avatar.facing) > 10. * math.pi / 180.:
            # turn towards
            self.unset_moving_forward();
            self.unset_attacking();
            if utils.compare_angles(angle_betw, self.avatar.facing) > 0:
                self.unset_turning_right();
                self.set_turning_left();
            else:
                self.unset_turning_left();
                self.set_turning_right();
        elif dist_betw > self.avatar.sword.length:
            #charge!
            self.unset_turning_left();
            self.unset_turning_right();
            self.set_attacking();
            self.set_moving_forward();
        else:
            self.unset_turning_left();
            self.unset_turning_right();
            self.unset_moving_forward();
            self.set_attacking();
            
        Player.update(self);
            
        
